/*
 * Generated by MyEclipse Struts
 * Template path: templates/java/JavaClass.vtl
 */
package gov.va.med.vistalink.management.action;

import java.util.Calendar;
import java.util.Date;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import gov.va.med.vistalink.adapter.cci.VistaLinkResourceException;
import gov.va.med.vistalink.adapter.config.Connector;
import gov.va.med.vistalink.adapter.spi.ConnectorConfigurator;
import gov.va.med.vistalink.institution.IPrimaryStationRules;
import gov.va.med.vistalink.institution.InstitutionMappingBadStationNumberException;
import gov.va.med.vistalink.institution.PrimaryStationRulesFactory;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.apache.struts.Globals;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;
import org.apache.struts.action.DynaActionForm;

/**
 * Struts action to save changes to a configuration entry.
 * 
 */
public class ConfiguratorSaveAction extends VistaLinkActionBase {

	/*
	 * Regex pattern for valid JNDI names for connectors. Used to prevent illegal JNDI names, also
	 * JNDI names that are hard to include in hyperlinks (e.g., containing '&').
	 */
	static Pattern jndiNamePattern;
	static {
		jndiNamePattern = Pattern.compile("[-_/()\\[\\]a-zA-Z0-9]+");
	}
	
	private static final Logger logger = Logger.getLogger(ConfiguratorSaveAction.class);

	/**
	 * Method execute
	 * @param mapping
	 * @param actionForm
	 * @param request
	 * @param response
	 * @return ActionForward
	 */
	public ActionForward execute(ActionMapping mapping, ActionForm actionForm, HttpServletRequest request,
			HttpServletResponse response) {

		// security check
		try {
			checkIsUserInRole(request, DEPLOYER_OR_ADMIN_OR_OPERATOR);
		} catch (SecurityException e) {
			logger.warn("Access attempt without sufficient privileges" + e.getMessage());
			ActionErrors errors = getGlobalActionErrors(request);
			errors.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("configurator.nopriv", new Object[] { e.getMessage() }));
			request.setAttribute(Globals.ERROR_KEY, errors);
			return mapping.findForward("nopriv");
		}
		
		logger.debug("in ConfiguratorSaveAction");
		addPageRenderedTime(request);
		addSessionTimeoutWarning(request);
		request.setAttribute("definitionLabel", definitionLabel);
		DynaActionForm form = (DynaActionForm) actionForm;

		if (isCancelled(request)) {
			logger.debug("request cancelled, forwarding to cancel");
			return mapping.findForward("cancel");
		}

		// check if deletion requested
		String submitValue = form.getString("submitValue");
		logger.debug("submitValue: " + submitValue);
		if ("delete".equalsIgnoreCase(submitValue)) {
			request.setAttribute("connectorForm", form);
			return mapping.findForward("confirmDelete");
		}

		boolean isNew = ((Boolean) form.get("new")).booleanValue();
		boolean isSaved = false;

		// "pre"-validate manually, because automatic struts validation (via form bean validation method) doesn't work
		// in BEA console
		ActionErrors errors = validate(mapping, request, form);
		if (errors != null) {

			logger.debug("found " + errors.size() + " validation problems.");
			request.setAttribute(Globals.ERROR_KEY, errors);
			// preserve status of new
			if (isNew)
				request.setAttribute("isNew", Boolean.TRUE);

		} else {
			// try saving if passed pre-validation
			String jndiName = form.getString("jndiName");
			ConnectorConfigurator cc = new ConnectorConfigurator();
			Connector conn = null;

			try {
				if (isNew) {
					conn = cc.addConnector(jndiName);
					logger.debug("'added' " + jndiName);
				} else {
					conn = cc.getConnectorByName(jndiName);
					logger.debug("retrieved " + jndiName);
				}
				logger.debug("retrieving connector values from form");
				String accessCode = form.getString("accessCode");
				if (!ConfiguratorRetrieveAction.AV_MASK.equals(accessCode)) {
					cc.setEncryptedAccessCode(conn, accessCode);
				}
				conn.setAlwaysUseDefaultAsMin(((Boolean) form.get("alwaysUseDefaultAsMin")).booleanValue());
				conn.setEnabled(((Boolean) form.get("enabled")).booleanValue());
				conn.setIp(form.getString("ip"));

				try {
					conn.setPort(Integer.parseInt(form.getString("port")));
				} catch (NumberFormatException e) {
					// swallow
				}
				conn.setPrimaryStation(form.getString("primaryStation"));
				try {
					conn.setTimeout(Integer.parseInt(form.getString("timeout")));
				} catch (NumberFormatException e) {
					// swallow
				}
				Calendar c = Calendar.getInstance();
				c.setTimeInMillis(new Date().getTime());
				conn.setTimestamp(c);
				String verifyCode = form.getString("verifyCode");
				if (!ConfiguratorRetrieveAction.AV_MASK.equals(verifyCode)) {
					cc.setEncryptedVerifyCode(conn, verifyCode);
				}
				
				// make sure encryption up-to-date
				cc.updateEncryptionIfNeeded(conn);
				logger.debug("saving connector values");
				cc.save();

				// reload form values from CC file before redisplaying (keep fidelity to what is in file, after save)
				ConfiguratorRetrieveAction.setFormValuesFromConnector(form, conn);

				isSaved = true;
				request.setAttribute("isSaved", Boolean.TRUE); // to tell list page an entry was saved
				request.setAttribute("jndiName", jndiName); // for list page to say what entry was saved

			} catch (VistaLinkResourceException e) {

				logger.error("failure saving configuration: ", e);
				errors = getGlobalActionErrors(request);
				errors.add(Globals.ERROR_KEY, new ActionMessage("configurator.generalError", new Object[] {
						"saving entry", e.getMessage() }));
				// preserve status of new
				if (isNew)
					request.setAttribute("isNew", Boolean.TRUE);

			} catch (Exception e) {

				logger.error("failure saving configuration: ", e);
				errors = getGlobalActionErrors(request);
				errors.add(Globals.ERROR_KEY, new ActionMessage("configurator.generalError", new Object[] {
						"saving entry", e.getMessage() + " (administrator should check log file for more details.)" }));
				// preserve status of new
				if (isNew)
					request.setAttribute("isNew", Boolean.TRUE);
			}
		}

		request.setAttribute("connectorForm", form);
		logger.debug("set DynaActionForm as request attribute.");

		if ((errors == null) && (isSaved)) {
			// saved changes without error
			logger.debug("returning success.");
			return mapping.findForward("success");

		} else {
			// validation or save failure
			request.setAttribute(Globals.ERROR_KEY, errors);
			logger.debug("returning validateFailure.");
			return mapping.findForward("validateFailure");
		}
	}

	ActionErrors validate(ActionMapping mapping, HttpServletRequest request, DynaActionForm form) {

		logger.debug("in validate.");
		// ActionErrors actionErrors = super.validate(mapping, request);
		ActionErrors actionErrors = getGlobalActionErrors(request);

		Matcher jndiNameMatcher = jndiNamePattern.matcher(form.getString("jndiName"));
		if (!jndiNameMatcher.matches()) {
			actionErrors.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("configurator.noJndiName"));
		}

		// a/v checks
		if (!form.get("accessCode").equals(form.get("accessCodeCheck"))) {
			actionErrors.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("configurator.accessCodeCheck"));
		}
		if (!form.get("verifyCode").equals(form.get("verifyCodeCheck"))) {
			actionErrors.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("configurator.verifyCodeCheck"));
		}

		// port string-to-integer conversion, check
		String portStr = form.getString("port");
		int port = 0;
		if ((portStr != null) && (portStr.length() > 0)) {
			try {
				port = Integer.parseInt(portStr);
			} catch (NumberFormatException e) {
				actionErrors.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("configurator.formatPort"));
			}
		} else {
			actionErrors.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("configurator.formatPort"));
		}

		// timeout string-to-intenger conversion, check
		String timeoutStr = form.getString("timeout");
		if ((timeoutStr != null) && (timeoutStr.length() > 0)) {
			try {
				Integer.parseInt(timeoutStr);
			} catch (NumberFormatException e) {
				actionErrors.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("configurator.formatTimeout"));
			}
		} else {
			actionErrors.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("configurator.formatTimeout"));
		}

		// rules to check 'if enabled'
		if (((Boolean) form.get("enabled")).booleanValue()) {

			// validate primary station based on primary station 'rules' class
			String primaryStation = (form.getString("primaryStation")).trim();
			try {
				IPrimaryStationRules rules = PrimaryStationRulesFactory.getPrimaryStationRulesImplementation();
				rules.validatePrimaryStation(primaryStation);
			} catch (InstantiationException e) {
				logger.error("Could not load primary station rules class.", e);
			} catch (IllegalAccessException e) {
				logger.error("Could not load primary station rules class.", e);
			} catch (ClassNotFoundException e) {
				logger.error("Could not load primary station rules class.", e);
			} catch (InstitutionMappingBadStationNumberException e) {
				logger.warn("invalid station number entered by user: " + primaryStation);
				actionErrors.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("configurator.formatPrimaryStation",
						new Object[] { primaryStation, e.getMessage() }));
			}

			// validate 'required' properties
			if ((form.getString("accessCode")).trim().length() < 1) {
				actionErrors.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("configurator.noAccessCode"));
			}
			if ((form.getString("verifyCode")).trim().length() < 1) {
				actionErrors.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("configurator.noVerifyCode"));
			}
			if (primaryStation.length() < 1) {
				actionErrors.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("configurator.noPrimaryStation"));
			}
			if ((form.getString("ip")).trim().length() < 1) {
				actionErrors.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("configurator.noIp"));
			}
			if (port < 1) {
				actionErrors.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("configurator.noPort"));
			}
		}
		return actionErrors.size() > 0 ? actionErrors : null;
	}
}